float4x4 tWIT : WorldInverseTranspose;
float4x4 tWInv : WorldInverse;
float4x4 tVP :  ViewProjection;
float4x4 tW   : World;

float height <String uiname="Height";>;
float maxHeight <String uiname="Max height";>;
float hairTextureScale <String uiname="Hair texture scale factor";>;
float3 lightPos <String uiname="Light Pos";>;
float ka <String uiname="Ambient Coefficient";>;
float kd <String uiname="Diffuse Coefficient";>;
float ks <String uiname="Specular Coefficient";>;
float specPower <String uiname="Specular Power";>;
float3 eye <String uiname="Eye Point";>;
float minHairLengthThreshold <String uiname="Min Hair Length Threshold";>;


texture BaseTexture;
texture HairTexture;
texture OffsetTexture;

sampler2D SampHair = sampler_state
           {
	       Texture = <HairTexture>;
               MinFilter = Linear;
	       MagFilter = Linear;
	       MipFilter = Linear;
	       AddressU = Wrap;
               AddressV = Wrap;
           };
sampler2D SampBase = sampler_state
           {
	       Texture = <BaseTexture>;
               MinFilter = Linear;
	       MagFilter = Linear;
	       MipFilter = Linear;
	       AddressU = Wrap;
               AddressV = Wrap;
            };
sampler2D SampOffset = sampler_state
           {
	       Texture = <OffsetTexture>;
               MinFilter = Linear;
	       MagFilter = Linear;
	       MipFilter = Linear;
	       AddressU = Wrap;
               AddressV = Wrap;
            };


/* data passed from vertex shader to pixel shader */

struct VS2PS
    {
     float4 HPosition	: POSITION;
     float2 Texcd : TEXCOORD0;
     float3 Tangent : TEXCOORD1;
     float3 BiNormal : TEXCOORD2;
     float3 Normal : TEXCOORD3;
     float4 Pos	: TEXCOORD4;
    };

/*********** vertex shader ******/

VS2PS VS(float4 Position : POSITION, float2 Texcd : TEXCOORD0, float3 Normal : NORMAL, float3 Tangent : TANGENT, float3 BiNormal : BINORMAL)
{
             VS2PS OUT;
             float3 normal = normalize(Normal);

             OUT.Texcd =  Texcd;
             OUT.Tangent = Tangent;
             OUT.Normal = normal;
             OUT.BiNormal = BiNormal;

             Position = mul(Position, tW) + float4(mul(normal * height, tWIT).xyz, 0);
             OUT.Pos = Position;
             OUT.HPosition = mul(Position, tVP);
             return OUT;
}


/********* pixel shader ********/
float4 PS_basic_coloured_ct  (VS2PS IN) : COLOR
{
      float4 BaseTexColor;
      float3 hairTangent = tex2D(SampOffset, IN.Texcd) * 2 - 1;
      float2 offset = 0.2 * hairTangent.xy * (height / maxHeight);
      float3 HairColor = tex2D(SampHair, (IN.Texcd + offset) * hairTextureScale);
      float hairHeight = HairColor.r * maxHeight;
      float height2 = height + length(offset);   // approx
      
      if(hairHeight >= height2  && hairHeight > minHairLengthThreshold)
      {
          BaseTexColor = tex2D(SampBase, IN.Texcd + offset);
          
          float3 lightVector   = normalize(mul(float3x3(IN.Tangent, IN.BiNormal, IN.Normal), mul((lightPos - IN.Pos), (float3x3)tWInv)));
          float3 viewVector    = normalize(mul(float3x3(IN.Tangent, IN.BiNormal, IN.Normal), mul((eye - IN.Pos), (float3x3)tWInv)));
          float3 tangentVector = normalize(tex2D(SampOffset, IN.Texcd) * 2 - 1);

          float TdotL = dot(tangentVector, lightVector);
          float TdotE = dot(tangentVector, viewVector);
          float sinTL = sqrt(1 - TdotL * TdotL);
          float sinTE = sqrt(1 - TdotE * TdotE);
          BaseTexColor.xyz = ka * BaseTexColor.xyz + kd * sinTL * BaseTexColor.xyz + ks * pow(abs((TdotL * TdotE + sinTL * sinTE)), specPower).xxx;

          float t = 1.0f - HairColor.r;
          BaseTexColor.rgb *= 1.0f - 0.8 * pow(t, 1.6f);
          BaseTexColor.a = (1 - height2) / hairHeight;
      }
      else
          discard;

      return BaseTexColor;
}


/*************/

technique ColouredCubeTexturing
{
	pass p0
        {
                VertexShader = compile vs_1_1 VS();
                PixelShader = compile ps_3_0 PS_basic_coloured_ct ();
                AlphaBlendEnable = true;
                SrcBlend = SRCALPHA;
                DestBlend = INVSRCALPHA;
                BlendOp = ADD;
                SrcBlendAlpha = ONE;
                DestBlendAlpha = ONE;
                BlendOpAlpha = ADD;
                ZWriteEnable = true;
                CullMode = CCW;
                AlphaTestEnable = true;
                AlphaRef = 130;
                AlphaFunc = GREATER;
	}
}
/***************************** eof ***/
