float4x4 tWIT : WorldInverseTranspose;
float4x4 tWVP :  WorldViewProjection;
float4x4 tW   : World;

texture BaseTexture;
texture FinTexture;
texture OffsetTexture;

float maxHeight <String uiname="Max height";>;
float3 eyePoint<String uiname="Eye Point";>;
float hairTextureScale <String uiname="Hair texture scale factor";>;
float minHairLengthThreshold <String uiname="Min Hair Length Threshold";>;
float heightScale <String uiname="Height scale factor";>;

sampler2D SampFin = sampler_state
           {
	       Texture = <FinTexture>;
               MinFilter = Linear;
	       MagFilter = Linear;
	       MipFilter = Linear;
	       AddressU = Wrap;
               AddressV = Wrap;
           };

sampler2D SampBase = sampler_state
           {
	       Texture = <BaseTexture>;
               MinFilter = Linear;
	       MagFilter = Linear;
	       MipFilter = Linear;
	       AddressU = Wrap;
               AddressV = Wrap;
           };
sampler2D SampOffset = sampler_state
           {
	       Texture = <OffsetTexture>;
               MinFilter = Linear;
	       MagFilter = Linear;
	       MipFilter = Linear;
	       AddressU = Wrap;
               AddressV = Wrap;
           };


/* data passed from vertex shader to pixel shader */

struct VS2PS
{
    float4 HPosition	: POSITION;
    float2 Texcd : TEXCOORD0;
    float2 OriginalTexcd : TEXCOORD1;
    float3 TransNormal : TEXCOORD2;
    float3 TransPos : TEXCOORD3;
};

/*********** vertex shader ******/

VS2PS VS(float3 Position : POSITION, float3 Normal : NORMAL, float2 Texcd : TEXCOORD0, float2 OriginalTexcd : TEXCOORD1)
{
             VS2PS OUT;
             float3 normal = normalize(Normal);
             float3 offset = ((tex2Dlod(SampOffset, float4(OriginalTexcd, 0, 0)) * 2 - 1) );
             
             OUT.TransNormal = mul(normal, tWIT);

             OUT.Texcd =  Texcd;
             OUT.OriginalTexcd = OriginalTexcd;
                                                    // very bad approximation, but there is no better way
             Position = Position + normalize(normal + offset * 0.02) * maxHeight * heightScale * (1.0f - Texcd.y);   // make fins stand out a bit by +0.01f



             OUT.TransPos = mul(Position, tW);
             OUT.HPosition = mul( float4(Position, 1.0f), tWVP );
             return OUT;
}


/********* pixel shader ********/
float4 PS_basic_coloured_ct  (VS2PS IN) : COLOR
{
      float3 offset = ((tex2D(SampOffset, float4(IN.OriginalTexcd * hairTextureScale, 0, 0)) * 2 - 1) ) * 0.2f;
      
      float4 FinTexColor = tex2D(SampFin, IN.OriginalTexcd * hairTextureScale);

      float hairHeight = FinTexColor.r;
      float height2 = 1 - IN.Texcd.y + length(offset); // aprox

      if(hairHeight >= height2 && hairHeight * maxHeight > minHairLengthThreshold)
      {
        FinTexColor.rgb = tex2D(SampBase, IN.OriginalTexcd + (1-IN.Texcd.y) * offset.xy);
        float t = 1.0f - height2;
        FinTexColor.rgb *= 1.0f - 0.8 * pow(t, 1.6f);

        float dot = dot(normalize(IN.TransNormal), normalize(eyePoint - IN.TransPos));
        FinTexColor.a *=  abs(dot) - 0.5 > 1 ? 0 : -(abs(dot) - 1.1);
      }
      else
        discard;

      return FinTexColor;
}


/*************/

technique FurFin
{
	pass p0
        {
                VertexShader = compile vs_3_0 VS();
                PixelShader = compile ps_2_0 PS_basic_coloured_ct ();
                AlphaBlendEnable = true;
                SrcBlend = SRCALPHA;
                DestBlend = INVSRCALPHA;
                BlendOp = ADD;
                CullMode = NONE;
                ZWriteEnable = true;
                AlphaTestEnable = true;
                AlphaRef = 20;
                AlphaFunc = GREATER;
	}
}
/***************************** eof ***/
