float3x3 tWInv   : WorldInverse;

float3 Gravity = float3(0,1,0);
float3 Translation = float3(0,0,0);
float3 RotationalForce = float3(0,0,0);
float timeDiff;
float MoveTime = 0.3;
float stiffness;
float maxHeight <String uiname="Max Height";>;

texture OldOffsetMap;

texture HairTexture;

sampler2D SampHair = sampler_state
           {
	       Texture = <HairTexture>;
               MinFilter = Linear;
	       MagFilter = Linear;
	       MipFilter = Linear;
	       AddressU = Wrap;
               AddressV = Wrap;
           };

sampler2D SampOldOffsetMap = sampler_state
           {
	       Texture = <OldOffsetMap>;
               MinFilter = Linear;
	       MagFilter = Linear;
	       MipFilter = Linear;
	       AddressU = Wrap;
               AddressV = Wrap;
           };
           



/* data passed from vertex shader to pixel shader */

struct VS2PS
{
    float4 HPosition	: POSITION;
    float2 Texcd : TEXCOORD0;
    float3 ForceTransformed : TEXCOORD1;
};

/*********** vertex shader ******/

VS2PS VS(float3 Position : POSITION,  float2 Texcd : TEXCOORD0, float3 Normal : NORMAL, float3 Tangent : TANGENT, float3 BiNormal : BINORMAL)
{
             VS2PS OUT;
          
             OUT.Texcd.x = Texcd.x;
             OUT.Texcd.y = 1 - Texcd.y;

             
             float3 forceVec = cross(mul(RotationalForce, tWInv), Position) + mul(Gravity * stiffness, tWInv) + Translation * stiffness;

             OUT.ForceTransformed = mul(float3x3(Tangent, BiNormal, Normal), forceVec);

             OUT.HPosition = float4(Texcd * 2 - 1, 0, 1);
             return OUT;
}


/********* pixel shader ********/
float4 PS_basic_coloured_ct  (VS2PS IN) : COLOR
{
      float3 oldOffset = tex2D(SampOldOffsetMap, IN.Texcd);
      float hairLength = tex2D(SampHair, IN.Texcd).r * maxHeight;

      float shiftLen = length(IN.ForceTransformed) * 0.2;  // Force * 0.2 in every shader (textures only 8bit->exacter values)
      if(shiftLen > hairLength)
                  IN.ForceTransformed = IN.ForceTransformed * hairLength / shiftLen;
      float3 force = clamp(IN.ForceTransformed * 0.5 + 0.5, 0, 1);

      float timeDiff2 = timeDiff / MoveTime;
      float3 result = oldOffset * (1 - timeDiff2) + force * timeDiff2;

      return clamp(float4(result,  1), 0, 1);

}


/*************/

technique OffsetMap
{
	pass p0
        {
                VertexShader = compile vs_1_1 VS();
                PixelShader = compile ps_2_0 PS_basic_coloured_ct ();
                AlphaBlendEnable = false;
                CullMode = None;
	}
}
/***************************** eof ***/
